@@ -27,6 +27,7 @@ Points of discussion:
     - Conceivably you could have a subclass for Figshare, etc., not just Alyx
     - We have mode and query_type, stick to just one?
 """
+import collections.abc
 import concurrent.futures
 import warnings
 import logging
@@ -152,10 +153,17 @@ class One(ConversionMixin):
     def refresh_cache(self, mode='auto'):
         """Check and reload cache tables
 
-        :param mode:
-        :return: Loaded time
+        Parameters
+        ----------
+        mode : str
+            Options are 'local' (don't reload); 'refresh' (reload); 'auto' (reload if expired);
+            'remote' (don't reload)
+
+        Returns
+        -------
+        Loaded timestamp
         """
-        if mode in ('local', 'remote'):  # TODO maybe rename mode
+        if mode in ('local', 'remote'):
             pass
         elif mode == 'auto':
             if datetime.now() - self._cache['_meta']['loaded_time'] >= self.cache_expiry:
@@ -171,8 +179,15 @@ class One(ConversionMixin):
     def _download_datasets(self, dsets, **kwargs) -> List[Path]:
         """
         Download several datasets given a set of datasets
-        :param dsets: list of dataset dictionaries from an Alyx REST query OR list of URL strings
-        :return: local file path list
+
+        Parameters
+        ----------
+        dsets : list
+            List of dataset dictionaries from an Alyx REST query OR URL strings
+
+        Returns
+        -------
+        A local file path list
         """
         out_files = []
         if hasattr(dsets, 'iterrows'):
@@ -190,9 +205,17 @@ class One(ConversionMixin):
     def _download_dataset(self, dset, cache_dir=None, **kwargs) -> Path:
         """
         Download a dataset from an alyx REST dictionary
-        :param dset: single dataset dictionary from an Alyx REST query OR URL string
-        :param cache_dir (optional): root directory to save the data in (home/downloads by default)
-        :return: local file path
+
+        Parameters
+        ----------
+        dset : pandas.Series, dict, str
+            A single dataset dictionary from an Alyx REST query OR URL string
+        cache_dir : str, pathlib.Path
+            The root directory to save the data in (home/downloads by default)
+
+        Returns
+        -------
+        The local file path
         """
         pass  # pragma: no cover
 
@@ -303,14 +326,23 @@ class One(ConversionMixin):
          check_filesystem?
          This changes datasets frame, calls _update_cache(sessions=None, datasets=None) to
          update and save tables.  Download_datasets can also call this function.
+        TODO Remove clobber
 
-        :param datasets: A list or DataFrame of dataset records
-        :param offline: If false and Web client present, downloads the missing datasets from a
-        remote repository
-        :param update_exists: If true, the cache is updated to reflect the filesystem
-        :param clobber: If true and not offline, datasets are re-downloaded regardless of local
-        filesystem
-        :return: A list of file paths for the datasets (None elements for non-existent datasets)
+        Parameters
+        ----------
+        datasets : pandas.Series, pandas.DataFrame, list of dicts
+            A list or DataFrame of dataset records
+        offline : bool, None
+            If false and Web client present, downloads the missing datasets from a remote
+            repository
+        update_exists : bool
+            If true, the cache is updated to reflect the filesystem
+        clobber : bool
+            If true and not offline, datasets are re-downloaded regardless of local filesystem
+
+        Returns
+        -------
+        A list of file paths for the datasets (None elements for non-existent datasets)
         """
         if offline or self.offline:
             files = []
@@ -380,7 +412,10 @@ class One(ConversionMixin):
     def list_subjects(self) -> List[str]:
         """
         List all subjects in database
-        :return: Sorted list of subject names
+
+        Returns
+        -------
+        Sorted list of subject names
         """
         return self._cache['sessions']['subject'].sort_values().unique()
 
@@ -393,12 +428,24 @@ class One(ConversionMixin):
         a list of all datasets is returned.  When details is false, a sorted array of unique
         datasets is returned (their relative paths).
 
-        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
-        details dict or Path
-        :param details: When true, a pandas DataFrame is returned, otherwise a numpy array of
-        relative paths (collection/revision/filename) - see one.alf.spec.describe for details.
-        :param collection: Filter by a given collection string
-        :return: Slice of datasets table or numpy array if details is False
+        Parameters
+        ----------
+        eid : str, UUID, pathlib.Path, dict
+            Experiment session identifier; may be a UUID, URL, experiment reference string
+            details dict or Path.
+        collection : str
+            The collection to which the object belongs, e.g. 'alf/probe01'.
+            This is the relative path of the file from the session root.
+            Supports asterisks as wildcards.
+        details : bool
+            When true, a pandas DataFrame is returned, otherwise a numpy array of
+            relative paths (collection/revision/filename) - see one.alf.spec.describe for details.
+        query_type : str
+            Query cache ('local') or Alyx database ('remote')
+
+        Returns
+        -------
+        Slice of datasets table or numpy array if details is False
         """
         datasets = self._cache['datasets']
         if not eid:
@@ -485,9 +532,17 @@ class One(ConversionMixin):
     def _filter_by_collection(datasets: pd.DataFrame, collection: str) -> pd.DataFrame:
         """
         Return a pandas datasets table containing records that match a given collection
-        :param datasets: A datasets cache table
-        :param collection: The collection name to match
-        :return: A slice of the input table, where all datasets match a given collection
+
+        Parameters
+        ----------
+        datasets : pandas.DataFrame
+            A datasets cache table
+        collection : str
+            The collection name to match.  Regular expressions permitted.
+
+        Returns
+        -------
+        A slice of the input table, where all datasets match a given collection
         """
         if collection is None:
             return datasets
@@ -516,24 +571,41 @@ class One(ConversionMixin):
         Load all attributes of an ALF object from a Session ID and an object name.  Any datasets
         with matching object name will be loaded.
 
-        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
-        details dict or Path
-        :param obj: The ALF object to load.  Supports asterisks as wildcards.
-        :param collection:  The collection to which the object belongs, e.g. 'alf/probe01'.
-        Supports asterisks as wildcards.
-        :param revision: The last revision before a given string (typically an ISO date).  If
-        None, the default dataset is returned (usually the most recent revision).
-        :param query_type: Query cache ('local') or Alyx database ('remote')
-        :param download_only: When true the data are downloaded and the file paths are returned
-        :param kwargs: Additional filters for datasets, including namespace and timescale
-        :return: An ALF bunch or if download_only is True, a list of Paths objects
+        Parameters
+        ----------
+        eid : str, UUID, pathlib.Path, dict
+            Experiment session identifier; may be a UUID, URL, experiment reference string
+            details dict or Path.
+        obj : str
+            The ALF object to load.  Supports asterisks as wildcards.
+        collection : str
+            The collection to which the object belongs, e.g. 'alf/probe01'.
+            This is the relative path of the file from the session root.
+            Supports asterisks as wildcards.
+        revision : str
+            The dataset revision (typically an ISO date).  If no exact match, the previous
+            revision (ordered lexicographically) is returned.  If None, the default revision is
+            returned (usually the most recent revision).  Regular expressions/wildcards not
+            permitted.
+        query_type : str
+            Query cache ('local') or Alyx database ('remote')
+        download_only : bool
+            When true the data are downloaded and the file path is returned.
+        kwargs : str
+            Additional filters for datasets, including namespace and timescale. For full list
+            see the one.alf.spec.describe function.
 
-        Examples:
-            load_object(eid, 'moves')
-            load_object(eid, 'trials')
-            load_object(eid, 'spikes', collection='.*probe01')
-            load_object(eid, 'spikes', namespace='ibl')
-            load_object(eid, 'spikes', timescale='ephysClock')
+        Returns
+        -------
+        An ALF bunch or if download_only is True, a list of Paths objects
+
+        Examples
+        --------
+        load_object(eid, 'moves')
+        load_object(eid, 'trials')
+        load_object(eid, 'spikes', collection='.*probe01')
+        load_object(eid, 'spikes', namespace='ibl')
+        load_object(eid, 'spikes', timescale='ephysClock')
         """
         query_type = query_type or self.mode
         datasets = self.list_datasets(eid, details=True, query_type=query_type)
@@ -557,9 +629,9 @@ class One(ConversionMixin):
         unique_objects = set(x[3] or '' for x in parts)
         unique_collections = set(x[0] or '' for x in parts)
         if len(unique_objects) > 1:
-            raise alferr.ALFMultipleObjectsFound('"' + '", "'.join(unique_objects) + '"')
+            raise alferr.ALFMultipleObjectsFound(*unique_objects)
         if len(unique_collections) > 1:
-            raise alferr.ALFMultipleCollectionsFound('"' + '", "'.join(unique_collections) + '"')
+            raise alferr.ALFMultipleCollectionsFound(*unique_collections)
 
         # For those that don't exist, download them
         offline = None if query_type == 'auto' else self.mode == 'local'
@@ -584,20 +656,42 @@ class One(ConversionMixin):
                      download_only: bool = False,
                      **kwargs) -> Any:
         """
-        Load a dataset for a given session id and dataset name
+        Load a single dataset for a given session id and dataset name
 
-        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
-        details dict or Path.
-        :param dataset: The ALF dataset to load.  Supports asterisks as wildcards.
-        :param collection:  The collection to which the object belongs, e.g. 'alf/probe01'.
-        For IBL this is the relative path of the file from the session root.
-        Supports asterisks as wildcards.
-        :param revision: The last revision before a given string (typically an ISO date).  If
-        None, the default dataset is returned (usually the most recent revision).
-        :param query_type: Query cache ('local') or Alyx database ('remote')
-        :param download_only: When true the data are downloaded and the file paths are returned
-        :param kwargs:
-        :return:
+        Parameters
+        ----------
+        eid : str, UUID, pathlib.Path, dict
+            Experiment session identifier; may be a UUID, URL, experiment reference string
+            details dict or Path.
+        dataset : str, dict
+            The ALF dataset to load.  May be a string or dict of ALF parts.  Supports asterisks as
+            wildcards.
+        collection : str
+            The collection to which the object belongs, e.g. 'alf/probe01'.
+            This is the relative path of the file from the session root.
+            Supports asterisks as wildcards.
+        revision : str
+            The dataset revision (typically an ISO date).  If no exact match, the previous
+            revision (ordered lexicographically) is returned.  If None, the default revision is
+            returned (usually the most recent revision).  Regular expressions/wildcards not
+            permitted.
+        query_type : str
+            Query cache ('local') or Alyx database ('remote')
+        download_only : bool
+            When true the data are downloaded and the file path is returned.
+
+        Returns
+        -------
+        Dataset or a Path object if download_only is true.
+
+        Examples
+        --------
+        intervals = one.load_dataset(eid, '_ibl_trials.intervals.npy')
+        intervals = one.load_dataset(eid, '*trials.intervals*')
+        filepath = one.load_dataset(eid '_ibl_trials.intervals.npy', download_only=True)
+        spike_times = one.load_dataset(eid 'spikes.times.npy', collection='alf/probe01')
+        old_spikes = one.load_dataset(eid, ''spikes.times.npy',
+                                      collection='alf/probe01', revision='2020-08-31')
         """
         datasets = self.list_datasets(eid, details=True, query_type=query_type or self.mode)
 
@@ -630,21 +724,34 @@ class One(ConversionMixin):
         first is the data (or file paths if download_data is false), the second is a list of
         meta data Bunches.  If assert_present is false, missing data will be returned as None.
 
-        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
-        details dict or Path.
-        :param datasets: The ALF datasets to load.  Supports asterisks as wildcards.
-        :param collections:  The collection(s) to which the object(s) belong, e.g. 'alf/probe01'.
-        For IBL this is the relative path of the file from the session root.
-        Supports asterisks as wildcards.
-        :param revisions: The last revision before a given string (typically an ISO date).  If
-        None, the default dataset is returned (usually the most recent revision).
-        :param query_type: Query cache ('local') or Alyx database ('remote')
-        :param assert_present: If true, missing datasets raises and error, otherwise None is
-        returned
-        :param download_only: When true the data are downloaded and the file paths are returned
-        :param kwargs:
+        Parameters
+        ----------
+        eid : str, UUID, pathlib.Path, dict
+            Experiment session identifier; may be a UUID, URL, experiment reference string
+            details dict or Path.
+        datasets : list of strings
+            The ALF datasets to load.  May be a string or dict of ALF parts.  Supports asterisks
+            as
+            wildcards.
+        collections : str, list
+            The collection(s) to which the object(s) belong, e.g. 'alf/probe01'.
+            This is the relative path of the file from the session root.
+            Supports asterisks as wildcards.
+        revisions : str, list
+            The dataset revision (typically an ISO date).  If no exact match, the previous
+            revision (ordered lexicographically) is returned.  If None, the default revision is
+            returned (usually the most recent revision).  Regular expressions/wildcards not
+            permitted.
+        query_type : str
+            Query cache ('local') or Alyx database ('remote')
+        assert_present : bool
+            If true, missing datasets raises and error, otherwise None is returned
+        download_only : bool
+            When true the data are downloaded and the file path is returned.
 
-        :return: Returns a list of data (or file paths) the length of datasets, and a list of
+        Returns
+        -------
+        Returns a list of data (or file paths) the length of datasets, and a list of
         meta data Bunches. If assert_present is False, missing data will be None
         """
 
@@ -953,29 +1060,42 @@ class OneAlyx(One):
                      query_type: str = None,
                      download_only: bool = False) -> Any:
         """
-        Load a single dataset from a Session ID and a dataset type.
-
-        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
-        details dict or Path.
-        :param dataset: The ALF dataset to load.  Supports asterisks as wildcards.
-        :param collection:  The collection to which the object belongs, e.g. 'alf/probe01'.
-        For IBL this is the relative path of the file from the session root.
-        Supports asterisks as wildcards.
-        :param revision: The last revision before a given string (typically an ISO date).  If
-        None, the default dataset is returned (usually the most recent revision).
-        :param query_type: Query cache ('local') or Alyx database ('remote')
-        :param download_only: When true the data are downloaded and the file path is returned.
-        :return: dataset or a Path object if download_only is true.
-
+        Load a single dataset from a Session ID and a dataset name.
         TODO This method may be removed once default dataset is added to Alyx serializer
 
-        Examples:
-            intervals = one.load_dataset(eid, '_ibl_trials.intervals.npy')
-            intervals = one.load_dataset(eid, '*trials.intervals*')
-            filepath = one.load_dataset(eid '_ibl_trials.intervals.npy', download_only=True)
-            spike_times = one.load_dataset(eid 'spikes.times.npy', collection='alf/probe01')
-            old_spikes = one.load_dataset(eid, ''spikes.times.npy',
-                                          collection='alf/probe01', revision='2020-08-31')
+        Parameters
+        ----------
+        eid : str, UUID, pathlib.Path, dict
+            Experiment session identifier; may be a UUID, URL, experiment reference string
+            details dict or Path.
+        dataset : str
+            The ALF dataset to load.  Supports asterisks as wildcards.
+        collection : str
+            The collection to which the object belongs, e.g. 'alf/probe01'.
+            This is the relative path of the file from the session root.
+            Supports asterisks as wildcards.
+        revision : str
+            The dataset revision (typically an ISO date).  If no exact match, the previous
+            revision (ordered lexicographically) is returned.  If None, the default revision is
+            returned (usually the most recent revision).  Regular expressions/wildcards not
+            permitted.
+        query_type : str
+            Query cache ('local') or Alyx database ('remote')
+        download_only : bool
+            When true the data are downloaded and the file path is returned.
+
+        Returns
+        -------
+        Dataset or a Path object if download_only is true.
+
+        Examples
+        --------
+        intervals = one.load_dataset(eid, '_ibl_trials.intervals.npy')
+        intervals = one.load_dataset(eid, '*trials.intervals*')
+        filepath = one.load_dataset(eid '_ibl_trials.intervals.npy', download_only=True)
+        spike_times = one.load_dataset(eid 'spikes.times.npy', collection='alf/probe01')
+        old_spikes = one.load_dataset(eid, ''spikes.times.npy',
+                                      collection='alf/probe01', revision='2020-08-31')
         """
         query_type = query_type or self.mode
         if query_type != 'remote':
@@ -1025,16 +1145,23 @@ class OneAlyx(One):
         Given an Alyx probe UUID string, returns the session id string and the probe label
         (i.e. the ALF collection)
 
-        :param pid: A probe UUID
-        :param query_type: Query mode, options include 'remote', and 'refresh'
-        :return: (experiment ID, probe label)
+        Parameters
+        ----------
+        pid : str, uuid.UUID
+            A probe UUID
+        query_type : str
+            Query mode - options include 'remote', and 'refresh'
+
+        Returns
+        -------
+        experiment ID, probe label
         """
         query_type = query_type or self.mode
         if query_type != 'remote':
             self.refresh_cache(query_type)
         if query_type == 'local' and 'insertions' not in self._cache.keys():
             raise NotImplementedError('Converting probe IDs required remote connection')
-        rec = self.alyx.rest('insertions', 'read', id=pid)
+        rec = self.alyx.rest('insertions', 'read', id=str(pid))
         return rec['session'], rec['name']
 
     def search(self, details=False, query_type=None, **kwargs):
@@ -1104,12 +1231,20 @@ class OneAlyx(One):
 
     def _download_dataset(self, dset, cache_dir=None, update_cache=True, **kwargs):
         """
-        Download a dataset from an alyx REST dictionary
-        :param dset: single dataset dictionary from an Alyx REST query OR URL string
-        :param cache_dir: root directory to save the data in (home/downloads by default)
-        :param update_cache: if true, the cache is updated when filesystem discrepancies are
-        encountered
-        :return: local file path
+        Download a dataset from an Alyx REST dictionary
+
+        Parameters
+        ----------
+        dset : dict, str
+            A single dataset dictionary from an Alyx REST query OR URL string
+        cache_dir : str, pathlib.Path
+            The root directory to save the data to (default taken from ONE parameters)
+        update_cache : bool
+            If true, the cache is updated when filesystem discrepancies are encountered
+
+        Returns
+        -------
+        A local file path
         """
         if isinstance(dset, str) and dset.startswith('http'):
             url = dset
@@ -1161,13 +1296,28 @@ class OneAlyx(One):
         """
         Downloads a single file from an HTTP webserver.  The webserver in question is set by the
         AlyxClient object.
-        :param url: an absolute or relative URL for a remote dataset
-        :param clobber: (bool: False) overwrites local dataset if any
-        :param offline:
-        :param keep_uuid:
-        :param file_size:
-        :param hash:
-        :return:
+
+        Parameters
+        ----------
+        url : str
+            An absolute or relative URL for a remote dataset
+        target_dir : str, pathlib.Path
+            The root directory to download file to
+        clobber : bool
+            If true, overwrites local dataset if any
+        offline : bool, None
+            If true, the file path is returned only if the file exists.  No download will take
+            place
+        keep_uuid : bool
+            If true, the UUID is not removed from the file name (default is False)
+        file_size : int
+            The expected file size to compare with downloaded file
+        hash : str
+            The expected file hash to compare with downloaded file
+
+        Returns
+        -------
+        The file path of the downloaded file
         """
         if offline is None:
             offline = self.mode == 'local'
@@ -1218,11 +1368,19 @@ class OneAlyx(One):
     @util.parse_id
     def eid2path(self, eid: str, query_type=None) -> util.Listable(Path):
         """
-        From an experiment id gets the local cache path
-        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
-        details dict or Path
-        :param query_type: if set to 'remote', will force database connection
-        :return: eid or list of eids
+        From an experiment ID gets the local session path
+
+        Parameters
+        ----------
+        eid : str, UUID, pathlib.Path, dict
+            Experiment session identifier; may be a UUID, URL, experiment reference string
+            details dict or Path.
+        query_type : str
+            If set to 'remote', will force database connection
+
+        Returns
+        -------
+        A session path or list of session paths
         """
         # first try avoid hitting the database
         mode = query_type or self.mode
@@ -1243,10 +1401,18 @@ class OneAlyx(One):
     @util.refresh
     def path2eid(self, path_obj: Union[str, Path], query_type=None) -> util.Listable(Path):
         """
-        From a local path, gets the experiment id
-        :param path_obj: local path or list of local paths
-        :param query_type: if set to 'remote', will force database connection
-        :return: eid or list of eids
+        From a local path, gets the experiment ID
+
+        Parameters
+        ----------
+        path_obj : str, pathlib.Path, list
+            Local path or list of local paths
+        query_type : str
+            If set to 'remote', will force database connection
+
+        Returns
+        -------
+        An eid or list of eids
         """
         # If path_obj is a list recurse through it and return a list
         if isinstance(path_obj, list):
@@ -1285,9 +1451,17 @@ class OneAlyx(One):
     def path2url(self, filepath, query_type=None):
         """
         Given a local file path, returns the URL of the remote file.
-        :param filepath: A local file path
-        :param query_type: if set to 'remote', will force database connection
-        :return: A URL string
+
+        Parameters
+        ----------
+        filepath : str, pathlib.Path
+            A local file path
+        query_type : str
+            If set to 'remote', will force database connection
+
+        Returns
+        -------
+        A URL string
         """
         query_type = query_type or self.mode
         if query_type != 'remote':
@@ -1301,18 +1475,32 @@ class OneAlyx(One):
             raise alferr.ALFObjectNotFound(f'File record for {filepath} not found on Alyx')
 
     @util.parse_id
-    def datasets_from_type(self, eid, dataset_type, full=False):
+    def type2datasets(self, eid, dataset_type, details=False):
         """
         Get list of datasets belonging to a given dataset type for a given session
-        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
-        details dict or Path
-        :param dataset_type: A dataset type, e.g. camera.times
-        :param full: If True, a dictionary of details is returned for each dataset
-        :return: A list of datasets belonging to that session's dataset type
+
+        Parameters
+        ----------
+        eid : str, UUID, pathlib.Path, dict
+            Experiment session identifier; may be a UUID, URL, experiment reference string
+            details dict or Path.
+        dataset_type : str, list
+            An Alyx dataset type, e.g. camera.times or a list of dtypes
+        details : bool
+            If True, a datasets DataFrame is returned
+
+        Returns
+        -------
+        A numpy array of data
         """
-        restriction = f'session__id,{eid},dataset_type__name,{dataset_type}'
-        datasets = self.alyx.rest('datasets', 'list', django=restriction)
-        return datasets if full else [d['name'] for d in datasets]
+        if isinstance(dataset_type, str):
+            restriction = f'session__id,{eid},dataset_type__name,{dataset_type}'
+        elif isinstance(dataset_type, collections.abc.Sequence):
+            restriction = f'session__id,{eid},dataset_type__name__in,{dataset_type}'
+        else:
+            raise TypeError('dataset_type must be a str or str list')
+        datasets = util.datasets2records(self.alyx.rest('datasets', 'list', django=restriction))
+        return datasets if details else datasets['rel_path'].sort_values().values
 
     def dataset2type(self, dset):
         """Return dataset type from dataset"""
@@ -1341,8 +1529,9 @@ class OneAlyx(One):
         datasets = self.list_datasets(eid) if eid else self._cache['datasets']
         # Get ID of fist matching dset
         for idx, rel_path in datasets['rel_path'].items():
-            if dset_name in rel_path:
+            if rel_path.endswith(dset_name):
                 return idx
+        raise ValueError(f'Dataset {dset_name} not found in cache')
 
     @util.refresh
     @util.parse_id
